#include "CyanBusCRC.hpp"

// C++ system level
#include <cinttypes>

// ESP32 specific
#include "esp_log.h"
#include "esp32/rom/crc.h"

// project specific
// #include "LambdaTask.hpp"


static const char* TAG = "CyanBus";

namespace SiliconTorch {

  namespace Impl {


    // Checks CyanBus CRC
    // Note: The last 4 bytes of buffer contain the CRC to check against and are included in the given length
    bool checkCRC(uint8_t* buffer, uint32_t length) {

      if (length < 4) {
        ESP_LOGE(TAG, "CRC calculation failed: Input buffer too small! Got bytes[ %d ] but needs bytes[ >= 4 ]", length);
        return false;
      }

      // uint32_t _init = 0x00000000;
      uint32_t _init = 0xFFFFFFFF;
      uint32_t calculatedCRC = crc32_be(_init, buffer, length - 4);
      uint32_t packetCRC = reinterpret_cast<uint32_t*>(&buffer[length - 4])[0];

      ESP_LOGW("CheckCRC", "packetCRC = 0x%X   calcCRC = 0x%X", packetCRC, calculatedCRC);

      return packetCRC == calculatedCRC;
    }


  }
}
